#!/usr/local/bin/perl

; $::RCSID =  q$Id: expire.pl,v 1.2 1997/02/06 16:52:46 issei Rel $;
# expire.pl  --  Expire old files like NetNews system.
#
# Copyright (C) 1997
# Issei Suzuki <p9424109@edu1.phys.titech.ac.jp>  All rights reserved.

use strict;
use English;
use Getopt::Std;

&main();

sub main () {
    my ($expire, $config, $find, $dir);

    # Analize command line options.
    unless (&getopts('cdf:')) {
	&usage();
    }
    $::CheckOnly = 1 if $::opt_c;
    $::Debug = 1 if $::opt_d;
    $find = $::opt_f ? $::opt_f : "/usr/bin/find";
    &usage unless (-x $find);

    unless ($config = shift(@ARGV)) {
	&usage();
    }

    # Start :)
    $expire = Expire::new($config, $find);
    foreach $dir ($expire->getDirectory()) {
	$expire->expire($dir);
    }
    exit(0);
}

sub usage () {
    my $ver = $::RCSID =~ /,v ([\d.]+)/ ? $1 : 'unkown';

    print STDERR <<"EOF";
Usage: expire.pl [-c] [-d] [-f FINDPROG] CONFIGFILE

  -c  Check only (Do not remove).
  -d  Debug mode.
  -f  Specify 'find' command (Default is '/usr/bin/find').

  expire.pl $ver
  Copyright (C) 1997  Issei Suzuki  All rights reserved.

EOF
    exit(8);
}

######################################################################
package Expire;

use strict;
use English;
use POSIX qw(:limits_h);

sub new () {
    my ($config, $find) = @_;
    my ($me, $dir) = undef;
    local($_);

    $me = {
	   find   => $find,		# find command
	   dirs   => [			# @DIRNAME
		     ],
	   conf   => {			# 'DIRNAME' => configuration
		     },
	   expire => {			# {'some'}{'where'}{'dir'} => expire day
		     },
	  };

    unless (open(F, $config)) {
	die "Error: $config : $ERRNO\n";
    }
    # analize configuration file
    while (<F>) {
	next unless(&cut_comment(\$_));
	if (m#//Directory:(.+)#) {
	    $dir = $1;
	    push(@{$me->{'dirs'}}, $dir);
	    print STDERR "Debug: new : Directory [$dir]\n" if $::Debug;
	}
	elsif ($dir) {
	    push(@{$me->{'conf'}{$dir}}, $_);
	    print STDERR "Debug: new :   => [$_]\n" if $::Debug;
	}
	else {				# not defined $dir
	    print STDERR "Error: $config at line $NR\n";
	    print STDERR "You must specifty 'directory' first.\n";
	    exit(6);
	}
    }
    close(F);

    bless $me;
    $me->setExpireDay();
    $me;
}

sub cut_comment (\$) {
    my $ptr = shift;
    chomp($$ptr);
    $$ptr =~ s/^([^\#]*)\#.*/$1/;
    $$ptr =~ s/\s*//g;

    $$ptr;
}

sub getDirectory ($) {
    my $me = shift;
    @{$me->{'dirs'}};
}


# Translate 'expire control' table into hasn reference.
# Are you confused what I do?  I know this code is trickey.
# Debug message maybe useful to understand. Run this script with '-d' option.
sub setExpireDay ($) {
    my $me = shift;
    my($dir, @subdir, $line, @splitLine, $day, $buf);
    local($_);

    foreach $dir (@{$me->{'dirs'}}) {
	foreach $line (@{$me->{'conf'}{$dir}}) {
	    if (scalar(@splitLine = split(':', $line)) != 2) {
		print STDERR "Warning: setExpireDay : Illigal format. Ignore..\n";
		print STDERR "         [$dir] => $line\n";
	    }
	    @subdir = split('/', $splitLine[0]);
	    if ($subdir[$#subdir] ne '.') {
		$subdir[$#subdir + 1] = '.';
	    }
	    $day = ($splitLine[1] =~ /never/i) ? LONG_MAX : $splitLine[1];
	    $buf =
	      '$me->{\'expire\'}{\''.$dir.'\'}{\''.join("'}{'", @subdir).'\'} = '.$day;
	    eval $buf;
	    print STDERR "Debug: setExpireDay : $buf\n" if $::Debug;
	}
	# Check '.' node that is neccesary.
	$buf ='$me->{\'expire\'}{\''.$dir.'\'}{\'.\'}';
	unless (defined(eval $buf)) {
	    print STDERR "Error: not defined [$buf]\n";
	    exit(8);
	}
    }
}

# $obj->getExpireDay('Dir', 'Subdir');
# Translate 'directory' structure into hasn reference, then check it.
sub getExpireDay ($$$) {
    my ($me, $dir, $subdir) = @_;
    my (@subdir, $buf, $day);

    @subdir = split('/', $subdir);
    push(@subdir, '.');

    # If the rule you apply to the entry is not existed, use the rule
    # apply to the parent entry.
    while (scalar(@subdir) > 0) {
	$buf = '$me->{\'expire\'}{\''.$dir.'\'}{\''.join("'}{'", @subdir).'\'}';
	print STDERR "Debug: getExpireDay : $buf\n" if $::Debug;
	if ($day = eval $buf) {
	    print STDERR "       found! $day\n" if $::Debug;
	    last;
	}
	# last element must be always '.'
	splice(@subdir, -2, 1);
    }
    $day;
}


sub expire ($$$) {
    my ($me, $dir) = @_;
    my ($subdir, $day);

    unless (-x $me->{'find'}) {
	die "Error: $me->{'find'} : Cannot execute.";
    }
    unless (open(FIND, "$me->{'find'} $dir -type d |")) {
	die "Error: $me->{'find'} : $ERRNO";
    }
    while ($subdir = <FIND>) {
	&full2rel(\$subdir, $dir);
	$day = $me->getExpireDay($dir, $subdir);
	if ($::Debug) {
	    print "[expire] $dir/$subdir : $day\n";
	}
	else {
	    &cleanDir("$dir/$subdir", $day);
	}
    }
}

# Rewrite 'fill path' notation into 'ralative path' notation.
sub full2rel (\$$) {
    my ($subdir, $base) = @_;
    chomp($$subdir);
    $base =~ s/(\W)/\\$1/g;
    $$subdir =~ s%^$base/?%%;
    unless ($$subdir) {
	$$subdir = ".";
    }
}

# &cleanDir(DIR, DAY);
sub cleanDir ($$) {
    my ($dir, $day) = @_;
    my ($file, $path);

    unless (opendir(D, $dir)) {
	print STDERR "Warning: $dir : $ERRNO. Skip...\n";
	return;
    }
    print "[$dir]\n";

    while ($file = readdir(D)) {
	$path = "$dir/$file";
	if ((-f $path) && (-M $path > $day)) {
	    print " $file";
	    print STDERR "Expire: $path\n" if $::Debug;
	    next if $::CheckOnly;
	    unless (unlink($path)) {
		die "Error: $path : $ERRNO";
	    }
	}
    }
    close(D);
    print "\n\n";
}
